@@ -1,6 +1,6 @@
 /*
 
-  $Id: cfgreader.c,v 1.91 2008/06/21 12:07:32 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -459,7 +459,7 @@ struct gn_cfg_header *cfg_memory_read(const char **lines)
 		}
 
 		/* Line not part of any heading */
-		fprintf(stderr, _("Orphaned line: %s\n"), line);
+		fprintf(stderr, _("Orphaned line: %s\nIf in doubt place this line into [global] section.\n"), line);
 
 		free(buf);
 	}
@@ -751,6 +751,8 @@ static gn_error cfg_psection_load(gn_config *cfg, const char *section, const gn_
 			fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "connection", val);
 			return GN_ERR_NOCONNECTION;
 		}
+		if (!strcmp("pcsc", val))
+			fprintf(stderr, "WARNING: %s=%s is deprecated and will soon be removed. Use %s=%s instead.\n", "connection", val, "connection", "libpcsclite");
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "initlength")))
@@ -930,6 +932,12 @@ static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 	char *val;
 	gn_error error;
 
+	error = gn_lib_init();
+	if (error != GN_ERR_NONE) {
+		fprintf(stderr, _("Failed to initialize libgnokii.\n"));
+		return error;
+	}
+
 	if (file == NULL && lines == NULL) {
 		fprintf(stderr, _("Couldn't open a config file or memory.\n"));
 		return GN_ERR_NOCONFIG;
