@@ -1,6 +1,6 @@
 /*
 
-  $Id: unixserial.c,v 1.47 2008/01/09 13:09:47 ladis Exp $
+  $Id$
 
   G N O K I I
 
@@ -93,18 +93,9 @@ static int cfsetspeed(struct termios *t, int speed)
 struct termios serial_termios;
 
 /* Script handling: */
-
 static void device_script_cfgfunc(const char *section, const char *key, const char *value)
 {
-#if !defined(__svr4__)
-#  if defined(HAVE_SETENV)
-	setenv(key, value, 1 /* overwrite */); /* errors ignored */
-#  else
-	char *str;
-	asprintf(&str, "%s=%s", key, value);
-	putenv(str);
-#  endif
-#endif
+	setenv(key, value, 1); /* errors ignored */
 }
 
 int device_script(int fd, const char *section, struct gn_statemachine *state)
