@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-sms.c,v 1.181 2008/04/23 20:12:04 dforsi Exp $
+  $Id$
 
   G N O K I I
 
@@ -57,21 +57,6 @@ struct sms_udh_data {
 
 #define MAX_SMS_PART 140
 
-/* These are positions in the following (headers[]) table. */
-#define SMS_UDH_CONCATENATED_MESSAGES  1
-#define SMS_UDH_RINGTONES              2
-#define SMS_UDH_OPERATOR_LOGOS         3
-#define SMS_UDH_CALLER_LOGOS           4
-#define SMS_UDH_MULTIPART_MESSAGES     5
-#define SMS_UDH_WAP_VCARD              6
-#define SMS_UDH_WAP_CALENDAR           7
-#define SMS_UDH_WAP_VCARDSECURE        8
-#define SMS_UDH_WAP_VCALENDARSECURE    9
-#define SMS_UDH_VOICE_MESSAGES        10
-#define SMS_UDH_FAX_MESSAGES          11
-#define SMS_UDH_EMAIL_MESSAGES        12
-#define SMS_UDH_WAP_PUSH              13
-
 /* User data headers */
 static struct sms_udh_data headers[] = {
 	{ 0x00, "" },
@@ -158,7 +143,7 @@ static char *sms_timestamp_print(u8 *number)
 	memset(buffer, 0, LOCAL_DATETIME_MAX_LENGTH);
 
 	/* Ugly hack, but according to the GSM specs, the year is stored
-         * as the 2 digit number. */
+	 * as the 2 digit number. */
 	if ((10*(number[0] & 0x0f) + (number[0] >> 4)) < 70)
 		snprintf(buffer, sizeof(buffer), "20");
 	else
@@ -223,7 +208,7 @@ gn_timestamp *sms_timestamp_unpack(unsigned char *number, gn_timestamp *dt)
 	dt->year = 10 * (number[0] & 0x0f) + (number[0] >> 4);
 
 	/* Ugly hack, but according to the GSM specs, the year is stored
-         * as the 2 digit number. */
+	 * as the 2 digit number. */
 	if (dt->year < 70)
 		dt->year += 2000;
 	else
@@ -263,7 +248,7 @@ unsigned char *sms_timestamp_pack(gn_timestamp *dt, unsigned char *number)
 		return number;
 
 	/* Ugly hack, but according to the GSM specs, the year is stored
-         * as the 2 digit number. */
+	 * as the 2 digit number. */
 	if (dt->year < 2000)
 		dt->year -= 1900;
 	else
@@ -437,7 +422,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
  * @output: room for the encoded text
  * @length: decoded text length
  * @size: encoded text length
- * @udhlen: udh length 
+ * @udhlen: udh length
  * @dcs: data coding scheme
  *
  * This function decodes either 7bit or 8bit or Unicode text to the
@@ -457,10 +442,14 @@ static gn_error sms_data_decode(unsigned char *message, unsigned char *output, u
 			memcpy(output, message + udhlen, length);
 		/* 7bit SMS */
 		} else {
+			char *aux;
+
 			dprintf("Default Alphabet\n");
 			length = length - (udhlen * 8 + ((7-(udhlen%7))%7)) / 7;
-			char_7bit_unpack((7-udhlen)%7, size, length, message, output);
-			char_ascii_decode(output, output, length);
+			aux = calloc(length + 1, 1);
+			char_7bit_unpack((7-udhlen)%7, size, length, message, aux);
+			char_default_alphabet_decode(output, aux, length);
+			free(aux);
 		}
 	}
 	dprintf("%s\n", output);
@@ -680,6 +669,7 @@ static gn_error sms_pdu_decode(gn_sms_raw *rawsms, gn_sms *sms)
 	sms->user_data[0].dr_status = GN_SMS_DR_Status_None;
 	switch (sms->type) {
 	case GN_SMS_MT_DeliveryReport:
+	case GN_SMS_MT_StatusReport:
 		sms_status(rawsms->report_status, sms);
 		break;
 	case GN_SMS_MT_PictureTemplate:
@@ -778,20 +768,22 @@ gn_error gn_sms_pdu2raw(gn_sms_raw *rawsms, unsigned char *pdu, int pdu_len, int
 	} \
 	memcpy(rawsms->remote_number, pdu + offset, l); \
 	offset += l; \
-	}
+}
 
 #define COPY_USER_DATA(pdu, offset) { \
 	rawsms->length = pdu[offset++]; \
 	rawsms->user_data_length = rawsms->length; \
-	if (rawsms->udh_indicator) \
-		rawsms->user_data_length -= pdu[offset] + 1; \
-	if (pdu_len - offset > 1000) { \
-		dprintf("Phone gave as poisonous (too short?) reply, either phone went crazy or communication went out of sync\n"); \
-		ret = GN_ERR_INTERNALERROR; \
-		goto out; \
+	if (rawsms->length > 0) { \
+		if (rawsms->udh_indicator) \
+			rawsms->user_data_length -= pdu[offset] + 1; \
+		if (pdu_len - offset > GN_SMS_USER_DATA_MAX_LENGTH) { \
+			dprintf("Phone gave as poisonous (too short?) reply, either phone went crazy or communication went out of sync\n"); \
+			ret = GN_ERR_INTERNALERROR; \
+			goto out; \
+		} \
+		memcpy(rawsms->user_data, pdu + offset, pdu_len - offset); \
 	} \
-	memcpy(rawsms->user_data, pdu + offset, pdu_len - offset); \
-	}
+}
 
 	gn_error ret = GN_ERR_NONE;
 	unsigned int l, extraoffset, offset = 0;
@@ -815,6 +807,7 @@ gn_error gn_sms_pdu2raw(gn_sms_raw *rawsms, unsigned char *pdu, int pdu_len, int
 	rawsms->report              = 0;
 
 	/* NOTE: offsets are taken from subclause 9.2.3 "Definition of the TPDU parameters"
+	   of 3GPP TS 23.040 version 7.0.1 Release 7 == ETSI TS 123 040 V7.0.1 (2007-03)
 	   *not* from the tables in subclause 9.2.2 "PDU Type repertoire at SM-TL" which ignore unused bits
 	*/
 
@@ -837,7 +830,6 @@ gn_error gn_sms_pdu2raw(gn_sms_raw *rawsms, unsigned char *pdu, int pdu_len, int
 	case 0x03:
 		dprintf("Reserved TP-MTI found\n");
 		return GN_ERR_INTERNALERROR;
-		break;
 	}
 	switch (rawsms->type) {
 	case GN_SMS_MT_Deliver:
@@ -931,22 +923,24 @@ gn_error gn_sms_pdu2raw(gn_sms_raw *rawsms, unsigned char *pdu, int pdu_len, int
 		memcpy(rawsms->time, pdu + offset, 7);
 		offset += 7;
 		/* TP-ST   TP-Status */
-		rawsms->report              = pdu[offset++];
+		rawsms->report_status       = pdu[offset++];
 		/* TP-PI   TP-Parameter-Indicator */
-		dprintf("TP-Parameter-Indicator: 0x%02x\n", pdu[offset++]);
 		parameter_indicator         = pdu[offset];
-		/* handle the "extension bit" skipping the following octects, if any (see 9.2.3.27 TP-Parameter-Indicator) */
-		while (pdu[offset++] & 0x80)
+		/* handle the "extension bit" skipping the following octects, if any (see 9.2.3.27 TP-Parameter-Indicator):
+		 *   The most significant bit in octet 1 and any other TP-PI octets which may be added later is reserved as an extension bit
+		 *   which when set to a 1 shall indicate that another TP-PI octet follows immediately afterwards.
+		 */
+		while ((offset < pdu_len) && (pdu[offset++] & 0x80))
 			;
-		if (parameter_indicator & 0x01) {
+		if ((offset < pdu_len) && (parameter_indicator & 0x01)) {
 			/* TP-PID  TP-Protocol-Identifier */
 			rawsms->pid = pdu[offset++];
 		}
-		if (parameter_indicator & 0x02) {
+		if ((offset < pdu_len) && (parameter_indicator & 0x02)) {
 			/* TP-DCS  TP-Data-Coding-Scheme */
 			rawsms->dcs = pdu[offset++];
 		}
-		if (parameter_indicator & 0x04) {
+		if ((offset < pdu_len) && (parameter_indicator & 0x04)) {
 			/* TP-UDL  TP-User-Data-Length */
 			/* TP-UD   TP-User-Data */
 			COPY_USER_DATA(pdu, offset);
@@ -1135,7 +1129,7 @@ static gn_error sms_get_read(gn_data *data)
 		if (!found) {
 			dprintf("Found new (read) message. Will store it at #%i!\n", data->folder_stats[data->sms_folder->folder_id]->used);
 			dprintf("%i\n", data->sms_folder->locations[i]);
-			data->message_list[data->folder_stats[data->sms_folder->folder_id]->used][data->sms_folder->folder_id]->location = 
+			data->message_list[data->folder_stats[data->sms_folder->folder_id]->used][data->sms_folder->folder_id]->location =
 				data->sms_folder->locations[i];
 			data->message_list[data->folder_stats[data->sms_folder->folder_id]->used][data->sms_folder->folder_id]->status = GN_SMS_FLD_New;
 			data->folder_stats[data->sms_folder->folder_id]->used++;
@@ -1357,7 +1351,7 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 		case 2: rawsms->dcs |= 0xf1; break; /* Class 1 */
 		case 3: rawsms->dcs |= 0xf2; break; /* Class 2 */
 		case 4: rawsms->dcs |= 0xf3; break; /* Class 3 */
-		default: dprintf("What ninja-mutant class is this?\n"); break; 
+		default: dprintf("What ninja-mutant class is this?\n"); break;
 		}
 		if (sms->dcs.u.general.compressed) {
 			/* Compression not supported yet */
@@ -1432,6 +1426,7 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 				udh_length = 0;
 			switch (al) {
 			case GN_SMS_DCS_DefaultAlphabet:
+				dprintf("Default Alphabet\n");
 				size = char_7bit_pack((7 - (udh_length % 7)) % 7,
 						      sms->user_data[i].u.text,
 						      rawsms->user_data + offset,
@@ -1440,11 +1435,13 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 				rawsms->user_data_length = size + offset;
 				break;
 			case GN_SMS_DCS_8bit:
+				dprintf("8bit\n");
 				rawsms->dcs |= 0xf4;
 				memcpy(rawsms->user_data + offset, sms->user_data[i].u.text, sms->user_data[i].u.text[0]);
 				rawsms->user_data_length = rawsms->length = length + udh_length;
 				break;
 			case GN_SMS_DCS_UCS2:
+				dprintf("UCS-2\n");
 				rawsms->dcs |= 0x08;
 				length = char_unicode_encode(rawsms->user_data + offset, sms->user_data[i].u.text, length);
 				rawsms->user_data_length = rawsms->length = length + udh_length;
@@ -1533,7 +1530,11 @@ gn_error sms_prepare(gn_sms *sms, gn_sms_raw *rawsms)
 	}
 	/* Encoding the header */
 	rawsms->report = sms->delivery_report;
-		rawsms->remote_number[0] = char_semi_octet_pack(sms->remote.number, rawsms->remote_number + 1, sms->remote.type);
+	rawsms->remote_number[0] = char_semi_octet_pack(sms->remote.number, rawsms->remote_number + 1, sms->remote.type);
+	if (rawsms->remote_number[0] > GN_SMS_NUMBER_MAX_LENGTH) {
+		dprintf("Remote number length %d > %d\n", rawsms->remote_number[0], GN_SMS_NUMBER_MAX_LENGTH);
+		return GN_ERR_ENTRYTOOLONG;
+	}
 	rawsms->validity_indicator = GN_SMS_VP_RelativeFormat;
 	rawsms->validity[0] = 0xa9;
 
@@ -1575,9 +1576,30 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state);
  */
 GNOKII_API gn_error gn_sms_send(gn_data *data, struct gn_statemachine *state)
 {
+	int i;
 	gn_error error = GN_ERR_NONE;
+	/*
+	 * This is for long sms handling. There we have sequence:
+	 * 1.		gn_sms_send()
+	 * 1.1			rawsms = malloc()
+	 * 1.2			sms_send_long()
+	 * 1.2.1			gn_sms_send()
+	 * 1.2.1.1				rawsms = malloc() // original rawsms from 1.1 is lost here
+	 * 1.2.1.2				free(rawsms)
+	 * 1.2.2			gn_sms_send()
+	 * 1.2.2.1				rawsms = malloc()
+	 * 1.2.2.2				free(rawsms)
+	 * ...
+	 * 1.2.N			gn_sms_send()
+	 * 1.2.N.1				rawsms = malloc()
+	 * 1.2.N.2				free(rawsms)
+	 * 1.3			free(rawsms) // rawsms was freed here just above
+	 * Therefore we store old rawsms value on the entrance and restore it on exit.
+	 */
+	gn_sms_raw *old = data->raw_sms;
 
-	if (!data->sms) return GN_ERR_INTERNALERROR;
+	if (!data->sms)
+		return GN_ERR_INTERNALERROR;
 
 	data->raw_sms = malloc(sizeof(*data->raw_sms));
 	memset(data->raw_sms, 0, sizeof(*data->raw_sms));
@@ -1590,7 +1612,8 @@ GNOKII_API gn_error gn_sms_send(gn_data *data, struct gn_statemachine *state)
 		data->raw_sms->message_center[0] = data->raw_sms->message_center[0] / 2 + 1;
 
 	error = sms_prepare(data->sms, data->raw_sms);
-	if (error != GN_ERR_NONE) goto cleanup;
+	if (error != GN_ERR_NONE)
+		goto cleanup;
 
 	sms_dump_raw(data->raw_sms);
 	if (data->raw_sms->user_data_length > MAX_SMS_PART) {
@@ -1601,9 +1624,22 @@ GNOKII_API gn_error gn_sms_send(gn_data *data, struct gn_statemachine *state)
 
 	dprintf("Sending\n");
 	error = gn_sm_functions(GN_OP_SendSMS, data, state);
+	if (data->sms->parts == 0) {
+		data->sms->parts = 1;
+		/* This is not multipart SMS. For multipart reference is allocated in sms_send_long() */
+		data->sms->reference = calloc(1, sizeof(unsigned int));
+	}
+	
+	/* We send SMS parts from the first part to last. */
+	i = 0;
+	while (i < data->sms->parts-1 && data->sms->reference[i] != 0)
+		i++;
+	data->sms->reference[i] = data->raw_sms->reference;
+
 cleanup:
-	free(data->raw_sms);
-	data->raw_sms = NULL;
+	if (data->raw_sms)
+		free(data->raw_sms);
+	data->raw_sms = old;
 	return error;
 }
 
@@ -1617,7 +1653,8 @@ GNOKII_API int gn_sms_udh_add(gn_sms *sms, gn_sms_udh_type type)
 
 static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 {
-	int i, count, start, copied, total, isConcat = -1, max_sms_len = MAX_SMS_PART;
+	static int init = 0;
+	int i, j, k, size, count, start, copied, total, refnum, isConcat = -1, max_sms_len = MAX_SMS_PART;
 	gn_sms sms;
 	gn_sms_user_data ud[GN_SMS_PART_MAX_NUMBER];
 	gn_error error = GN_ERR_NONE;
@@ -1656,15 +1693,29 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 
 	/* We need to attach user data header to each part */
 	max_sms_len -= (data->sms->udh.length + 1);
+	dprintf("max_sms_len: %d\n", max_sms_len);
 	/* Count number of SMS to be sent */
-	count = (total + max_sms_len - 1)/ max_sms_len;
+	count = (total + max_sms_len - 1) / max_sms_len;
 	dprintf("Will need %d sms-es\n", count);
 
+	data->sms->parts = count;
+	data->sms->reference = calloc(count, sizeof(unsigned int));
+
 	start = 0;
 	copied = 0;
+
+	/* Generate reference number */
+	if (!init) {
+		time_t t;
+		time(&t);
+		srand(t);
+		init = 1;
+	}
+	refnum = (int)(255.0*rand()/(RAND_MAX+1.0));
+
 	for (i = 0; i < count; i++) {
-		dprintf("Sending sms #%d\n", i);
-		data->sms->udh.udh[isConcat].u.concatenated_short_message.reference_number = 0;
+		dprintf("Sending sms #%d (refnum: %d)\n", i+1, refnum);
+		data->sms->udh.udh[isConcat].u.concatenated_short_message.reference_number = refnum;
 		data->sms->udh.udh[isConcat].u.concatenated_short_message.maximum_number = count;
 		data->sms->udh.udh[isConcat].u.concatenated_short_message.current_number = i+1;
 		switch (data->sms->dcs.u.general.alphabet) {
@@ -1681,24 +1732,43 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 			memcpy(data->sms->user_data[0].u.text, ud[0].u.text+start, copied);
 			break;
 		case GN_SMS_DCS_UCS2:
+			/* We need to copy precisely not to cut character in the middle */
 			start += copied;
-			copied = (ud[0].length - start) % max_sms_len;
-			/* This is maximum size of what we can send */
-			if (copied == 0) {
-				copied = max_sms_len;
-			}
-			if (copied > max_sms_len / 2) {
-				copied = copied / 2;
-				if (copied < max_sms_len / 2)
-					copied = max_sms_len / 2;
-			}
-			if (i < count - 1 && copied % 2) {
-				copied--;
-			}
 			memset(&data->sms->user_data[0], 0, sizeof(gn_sms_user_data));
 			data->sms->user_data[0].type = ud[0].type;
-			data->sms->user_data[0].length = copied;
-			memcpy(data->sms->user_data[0].u.text, ud[0].u.text+start, copied);
+			/* We assume UTF8 input */
+			size = 1;
+#define C ud[0].u.text[start + j]
+			for (j = 0, k = 0; start + j < ud[0].length && k < max_sms_len / 2; j++) {
+				size--;
+				if (!size) {
+					/* Let's detect size of UTF8 char */
+					if (C >= 0 && C < 128)
+						size = 1;
+					else if (C >= 192 && C < 224)
+						size = 2;
+					else if (C >= 224 && C < 240)
+						size = 3;
+					else if (C >= 240 && C < 248)
+						size = 4;
+					else if (C >= 248 && C < 252)
+						size = 5;
+					else if (C >= 252 && C < 254)
+						size = 6;
+					else
+						dprintf("CHARACTER ENCODING ERROR\n");
+					/* Avoid cutting the character */
+					if (j + size > max_sms_len / 2) {
+						dprintf("DEBUG: break: %d %d %d\n", j, size, max_sms_len / 2);
+						break;
+					}
+					k++;
+				}
+				data->sms->user_data[0].u.text[j] = ud[0].u.text[start + j];
+			}
+#undef C
+			data->sms->user_data[0].length = copied = j;
+			dprintf("DEBUG: copied: %d\n", copied);
 			break;
 		default:
 			switch (ud[0].type) {
@@ -1711,6 +1781,7 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 			}
 			break;
 		}
+		dprintf("Text to be sent in this part: %s\n", data->sms->user_data[0].u.text);
 		error = gn_sms_send(data, state);
 		ERROR();
 	}
@@ -1733,7 +1804,7 @@ GNOKII_API gn_error gn_sms_save(gn_data *data, struct gn_statemachine *state)
 	dprintf("\tDate: %s\n", sms_timestamp_print(data->raw_sms->smsc_time));
 
 	if (data->sms->smsc.number[0] != '\0') {
-		data->raw_sms->message_center[0] = 
+		data->raw_sms->message_center[0] =
 			char_semi_octet_pack(data->sms->smsc.number, data->raw_sms->message_center + 1, data->sms->smsc.type);
 		if (data->raw_sms->message_center[0] % 2) data->raw_sms->message_center[0]++;
 		if (data->raw_sms->message_center[0])
@@ -1760,7 +1831,7 @@ cleanup:
 
 /* WAPPush functions */
 
-char *encode_attr_inline_string(char token, char *string, int *data_len) 
+char *encode_attr_inline_string(char token, char *string, int *data_len)
 {
 	char *data = NULL;
 	
@@ -1780,7 +1851,7 @@ char *encode_attr_inline_string(char token, char *string, int *data_len)
 	return data;
 }
 
-char *encode_indication(gn_wap_push *wp, int *data_len) 
+char *encode_indication(gn_wap_push *wp, int *data_len)
 {
 	char *data = NULL;
 	char *attr = NULL;
@@ -1854,7 +1925,7 @@ char *encode_si(gn_wap_push *wp, int *data_len)
 	return data;
 }
 
-GNOKII_API gn_error gn_wap_push_encode(gn_wap_push *wp) 
+GNOKII_API gn_error gn_wap_push_encode(gn_wap_push *wp)
 {
 	
 	char *data = NULL;
@@ -1880,7 +1951,7 @@ GNOKII_API gn_error gn_wap_push_encode(gn_wap_push *wp)
 	return GN_ERR_NONE;
 }
 
-GNOKII_API void gn_wap_push_init(gn_wap_push *wp) 
+GNOKII_API void gn_wap_push_init(gn_wap_push *wp)
 {
 
 	if (!wp) {
@@ -2006,8 +2077,18 @@ GNOKII_API char *gn_sms2mbox(gn_sms *sms, char *from)
 #ifdef ENABLE_NLS
 	loc = setlocale(LC_ALL, "C");
 #endif
-	CONCAT(buf, tmp, size, "From %s@%s %s", sms->remote.number, from, asctime(loctime));
-
+	switch (sms->status) {
+	case GN_SMS_Sent:
+	case GN_SMS_Unsent:
+		CONCAT(buf, tmp, size, "To %s@%s %s", sms->remote.number, from, asctime(loctime));
+		break;
+	case GN_SMS_Read:
+	case GN_SMS_Unread:
+	default:
+		CONCAT(buf, tmp, size, "From %s@%s %s", sms->remote.number, from, asctime(loctime));
+		break;
+	}
+	
 	tmp = calloc(MAX_DATE_LENGTH, sizeof(char));
 	if (!tmp)
 		goto error;
@@ -2017,7 +2098,18 @@ GNOKII_API char *gn_sms2mbox(gn_sms *sms, char *from)
 #endif
 	APPEND(buf, tmp, size);
 
-	CONCAT(buf, tmp, size, "From: %s@%s\n", sms->remote.number, from);
+	switch (sms->status) {
+	case GN_SMS_Sent:
+	case GN_SMS_Unsent:
+		CONCAT(buf, tmp, size, "To: %s@%s\n", sms->remote.number, from);
+		break;
+	case GN_SMS_Read:
+	case GN_SMS_Unread:
+	default:
+		CONCAT(buf, tmp, size, "From: %s@%s\n", sms->remote.number, from);
+		break;
+	}
+
 	CONCAT(buf, tmp, size, "X-GSM-SMSC: %s\n", sms->smsc.number);
 	CONCAT(buf, tmp, size, "X-GSM-Status: %s\n", status2str(sms->status));
 	CONCAT(buf, tmp, size, "X-GSM-Memory: %s\n", gn_memory_type2str(sms->memory_type));
