@@ -1,5 +1,5 @@
 /*
-  $Id: libfunctions.c,v 1.39 2008/01/03 21:19:30 dforsi Exp $
+  $Id$
 
   G N O K I I
 
@@ -30,7 +30,7 @@
 
   Goal of this code is to provide a binary compatible layer to access
   libgnokii.so functions from external programs which are not part
-  of this gnokii CVS package.
+  of this gnokii package.
   One of those external programs is e.g. the Mobile Phone import/export
   filter of the KDE Adressbook (kaddressbook).
   Helge Deller - April 2006
@@ -47,9 +47,38 @@
 #include "gnokii.h"
 #include "device.h"
 
+#ifdef ENABLE_NLS
+#  include <locale.h>
+#endif
+
 /* this macro sets the "lasterror" code */
 #define LASTERROR(state,nr)	((state->lasterror = nr)) /* do not delete the double brackets! */
 
+/**
+ * gn_lib_init:
+ *
+ * returns: GN_ERR_NONE on success, an error on failure
+ *
+ * Initializes the library for use by programs.
+ * Call this function before any other in this library.
+ */
+GNOKII_API gn_error gn_lib_init()
+{
+	static bool initialized = false;
+
+	if (initialized == true)
+		return GN_ERR_NONE;
+
+#ifdef ENABLE_NLS
+	setlocale (LC_ALL, "");
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	textdomain (GETTEXT_PACKAGE);
+#endif
+	initialized = true;
+
+	return GN_ERR_NONE;
+}
+
 GNOKII_API unsigned int gn_lib_version()
 {
 	/* return the library version number at compile time of libgnokii */
@@ -129,7 +158,7 @@ GNOKII_API gn_error gn_lib_phone_open( struct gn_statemachine *state )
 		state->lockfile = gn_device_lock(state->config.port_device);
 		if (state->lockfile == NULL) {
 			fprintf(stderr, _("Lock file error. Exiting.\n"));
-			return LASTERROR(state, GN_ERR_BUSY);
+			return LASTERROR(state, GN_ERR_LOCKED);
 		}
 	}
 
@@ -138,9 +167,11 @@ GNOKII_API gn_error gn_lib_phone_open( struct gn_statemachine *state )
 	if (error != GN_ERR_NONE) {
 		fprintf(stderr, _("Telephone interface init failed: %s\nQuitting.\n"),
 			gn_error_print(error));
-		/* gn_device_unlock frees state->lockfile */
+
+		/* gn_device_unlock() removes lockfile if it was created and frees state->lockfile */
 		gn_device_unlock(state->lockfile);
 		state->lockfile = NULL;
+
 		return LASTERROR(state, error);
 	}
 
@@ -152,11 +183,8 @@ GNOKII_API gn_error gn_lib_phone_close( struct gn_statemachine *state )
 	/* close phone connection */
 	gn_sm_functions(GN_OP_Terminate, NULL, state);
 
-	/* remove lockfile if it was created */
-	if (state->lockfile) {
-		/* gn_device_unlock frees state->lockfile */
-		gn_device_unlock(state->lockfile);
-	}
+	/* gn_device_unlock() removes lockfile if it was created and frees state->lockfile */
+	gn_device_unlock(state->lockfile);
 	state->lockfile = NULL;
 
 	return LASTERROR(state, GN_ERR_NONE);
@@ -467,6 +495,7 @@ static struct { gn_connection_type ct; const char *str; } connectiontypes[] = {
 	{ GN_CT_DKU2LIBUSB, "dku2libusb" },
 #endif
 #ifdef HAVE_PCSC
+	{ GN_CT_PCSC,       "libpcsclite" },
 	{ GN_CT_PCSC,       "pcsc" },
 #endif
 #ifndef WIN32
@@ -557,6 +586,7 @@ GNOKII_API gn_error gn_lib_search_one_connected_phone(struct gn_statemachine **s
 
 	/* not yet implemented */
 	free(*state);
-	return LASTERROR((*state), GN_ERR_UNKNOWNMODEL);
+	*state = NULL;
+	return GN_ERR_NOTIMPLEMENTED;
 }
 
