@@ -1,6 +1,6 @@
 /*
 
-  $Id: fake.c,v 1.23 2008/07/28 14:47:36 dforsi Exp $
+  $Id$
 
   G N O K I I
 
@@ -33,6 +33,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <time.h>
 
 #include "compat.h"
 #include "misc.h"
@@ -92,6 +93,7 @@ static gn_error at_sms_write(gn_data *data, struct gn_statemachine *state, char*
 {
 	unsigned char req[10240], req2[5120];
 	int length, tmp, offset = 0;
+	static int init = 0;
 
 	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
 
@@ -127,9 +129,46 @@ static gn_error at_sms_write(gn_data *data, struct gn_statemachine *state, char*
 	req[length * 2] = 0x1a;
 	req[length * 2 + 1] = 0;
 	fprintf(stdout, "%s\n", req);
+
+	/* Generate reference number */
+	if (!init) {
+		time_t t;
+		time(&t);
+		srand(t);
+		init = 1;
+	}
+	data->raw_sms->reference = (int)(255.0*rand()/(RAND_MAX+1.0));
 	return GN_ERR_NONE;
 }
 
+/* SMS-DELIVER */
+/* Text: Hello! vowels àèìòì euro */
+#define SMS "0791214365870921240B919999999999F90000902072129025401AC8329BFD0E81ECEF7B993D07FD0907C40154AECBDF2010"
+/* Text: € */
+/* #define SMS "0791214365870921240B919999999999F9000090907000920480029B32" */
+/* SMS-STATUS-REPORT */
+/* Status: delivered */
+/* #define SMS "07914366999999F906CD098186666666F89080709002818090807002336080009FD6E494" */
+
+static gn_error at_sms_get(gn_data *data, struct gn_statemachine *state)
+{
+	gn_error e = GN_ERR_NONE;
+	char *tmp;
+	int len;
+
+	if (!data || !data->raw_sms) {
+		e = GN_ERR_INTERNALERROR;
+		goto out;
+	}
+	len = strlen(SMS) / 2;
+	tmp = calloc(len, 1);   
+	hex2bin(tmp, SMS, len);
+	e = gn_sms_pdu2raw(data->raw_sms, tmp, len, GN_SMS_PDU_DEFAULT);
+	free(tmp);
+out:
+	return e;
+}
+
 static gn_error fake_functions(gn_operation op, gn_data *data, struct gn_statemachine *state)
 {
 	switch (op) {
@@ -139,6 +178,8 @@ static gn_error fake_functions(gn_operation op, gn_data *data, struct gn_statema
 		return GN_ERR_NONE;
 	case GN_OP_SendSMS:
 		return at_sms_write(data, state, "???");
+	case GN_OP_GetSMS:
+		return at_sms_get(data, state);
 	case GN_OP_GetSMSCenter:
 		return GN_ERR_NONE;
 	default:
