@@ -1,6 +1,6 @@
 /*
 
-  $Id: nokia.c,v 1.44 2007/11/07 18:28:19 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -22,21 +22,22 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2000 Hugh Blemings & Pavel Janik ml.
   Copyright (C) 2001 Manfred Jonsson
   Copyright (C) 2002 Ladis Michl, Markus Plail
   Copyright (C) 2002-2004 BORBELY Zoltan, Pawel Kot
 
-  This file provides useful functions for all phones
+  This file provides useful functions for all Nokia phones.
   See README for more details on supported mobile phones.
 
   The various routines are called pnok_(whatever).
 
 */
 
+#include "config.h"
+
 #include <string.h>
 
-#include "config.h"
 #include "compat.h"
 #include "gnokii-internal.h"
 #include "gnokii.h"
@@ -270,12 +271,14 @@ gn_error pnok_call_divert_incoming(int messagetype, unsigned char *message, int
 
 	/* FIXME: is this common between models? - bozo */
 	/* get prepaid info */
-	case 0x05:
-		n = char_7bit_unpack(0, message[7], sizeof(buf), message + 8, buf);
-		char_ascii_decode(buf, buf, n);
+	case 0x05: {
+		char *aux = calloc(sizeof(buf) + 1, 1);
+		n = char_7bit_unpack(0, message[7], sizeof(buf), message + 8, aux);
+		char_default_alphabet_decode(buf, aux, n);
+		free(aux);
 		dprintf("Message: Prepaid info received: \"%s\"\n", buf);
 		return GN_ERR_UNSOLICITED;
-
+	}
 	default:
 		return GN_ERR_UNHANDLEDFRAME;
 	}
