@@ -1,6 +1,6 @@
 /*
 
-  $Id: vcal.c,v 1.25 2008/06/24 09:16:53 hadess Exp $
+  $Id$
 
   G N O K I I
 
@@ -43,7 +43,7 @@ static const char gnokii_version[] = VERSION;
 #ifdef HAVE_LIBICAL
 /* needs gnokii-internal.h for string_base64() and utf8_encode() */
 #  include "gnokii-internal.h"
-#  include "ical.h"
+#  include <libical/ical.h>
 /* mother's little helpers */
 static int iterate_cal(icalcomponent *parent, int depth, int *count, icalcomponent **result, icalcomponent_kind kind);
 static const char *comp_get_category(icalcomponent *comp);
