@@ -1,6 +1,6 @@
 /*
 
-  $Id: vcard.c,v 1.33 2008/05/25 22:26:30 dforsi Exp $
+  $Id$
   
   G N O K I I
 
@@ -37,58 +37,92 @@
 #include "gnokii.h"
 #include "gnokii-internal.h"
 
+typedef struct {
+	char *str;
+	char *end;
+	unsigned int len;
+} vcard_string;
+
 /* Write a string to a file doing folding when needed (see RFC 2425) */
-static int gn_vcard_fprintf(FILE *f, const char *fmt, ...)
+static void vcard_append_printf(vcard_string *str, const char *fmt, ...)
 {
-	char buf[1024], *current;
-	size_t pos;
+	char buf[1024];
 	va_list ap;
+	int len, lines, l;
+	char *s = "\r\n";
 
 	va_start(ap, fmt);
 	vsnprintf(buf, sizeof(buf), fmt, ap);
 	va_end(ap);
 
-	for (pos = 1, current = buf; *current; pos++) {
-		if (pos == 76) {
-			fprintf(f, "\r\n ");
-			pos = 0;
-		}
-		fputc(*current++, f);
-	}
-	fprintf(f, "\r\n");
+	/* Number of lines needed */
+	lines = strlen(buf) / 76 + 1;
+	/* 3 characters for each line beyond the first one,
+	 * plus the length of the buffer
+	 * plus the line feed and the nul byte to finish it off */
+	len = (lines - 1) * 3 + strlen (buf) + 3;
 
-	/* FIXME: count also end of line chars? */
-	return current - buf;
+	/* The first malloc must have a nul byte at the end */
+	if (str->str)
+		str->str = realloc(str->str, len + str->len);
+	else
+		str->str = realloc(str->str, len + 1);
+	if (str->end == NULL)
+		str->end = str->str;
+	else
+		str->end = str->str + str->len;
+
+	for (l = 0; l < lines; l++) {
+		int to_copy;
+
+		to_copy = GNOKII_MIN(76, strlen (buf) - 76 * l);
+		memcpy (str->end,  buf + 76 * l, to_copy);
+		str->end = str->end + to_copy;
+		if (l != lines - 1) {
+			char *s = "\r\n ";
+			memcpy (str->end, s, 3);
+			str->end += 3;
+		}
+	}
+
+	memcpy (str->end, s, 2);
+	str->end += 2;
+	str->end[0] = '\0';
+
+	str->len = str->end - str->str;
 }
 
-GNOKII_API int gn_phonebook2vcard(FILE *f, gn_phonebook_entry *entry, char *location)
+GNOKII_API char * gn_phonebook2vcardstr(gn_phonebook_entry *entry)
 {
+	vcard_string str;
 	int i;
 	char name[2 * GN_PHONEBOOK_NAME_MAX_LENGTH];
 
-	gn_vcard_fprintf(f, "BEGIN:VCARD");
-	gn_vcard_fprintf(f, "VERSION:3.0");
+	memset(&str, 0, sizeof (str));
+
+	vcard_append_printf(&str, "BEGIN:VCARD");
+	vcard_append_printf(&str, "VERSION:3.0");
 	add_slashes(name, entry->name, sizeof(name), strlen(entry->name));
-	gn_vcard_fprintf(f, "FN:%s", name);
+	vcard_append_printf(&str, "FN:%s", name);
 
 	if (entry->person.has_person)
-		gn_vcard_fprintf(f, "N:%s;%s;%s;%s;%s",
+		vcard_append_printf(&str, "N:%s;%s;%s;%s;%s",
 			entry->person.family_name[0]        ? entry->person.family_name        : "",
 			entry->person.given_name[0]         ? entry->person.given_name         : "",
 			entry->person.additional_names[0]   ? entry->person.additional_names   : "",
 			entry->person.honorific_prefixes[0] ? entry->person.honorific_prefixes : "",
 			entry->person.honorific_suffixes[0] ? entry->person.honorific_suffixes : "");
 	else
-		gn_vcard_fprintf(f, "N:%s", name);
+		vcard_append_printf(&str, "N:%s", name);
 
-	gn_vcard_fprintf(f, "TEL;TYPE=PREF,VOICE:%s", entry->number);
-	gn_vcard_fprintf(f, "X-GSM-MEMORY:%s", gn_memory_type2str(entry->memory_type));
-	gn_vcard_fprintf(f, "X-GSM-LOCATION:%d", entry->location);
-	gn_vcard_fprintf(f, "X-GSM-CALLERGROUP:%d", entry->caller_group);
-	gn_vcard_fprintf(f, "CATEGORIES:%s", gn_phonebook_group_type2str(entry->caller_group));
+	vcard_append_printf(&str, "TEL;TYPE=PREF,VOICE:%s", entry->number);
+	vcard_append_printf(&str, "X-GSM-MEMORY:%s", gn_memory_type2str(entry->memory_type));
+	vcard_append_printf(&str, "X-GSM-LOCATION:%d", entry->location);
+	vcard_append_printf(&str, "X-GSM-CALLERGROUP:%d", entry->caller_group);
+	vcard_append_printf(&str, "CATEGORIES:%s", gn_phonebook_group_type2str(entry->caller_group));
 
 	if (entry->address.has_address)
-		gn_vcard_fprintf(f, "ADR;TYPE=HOME,PREF:%s;%s;%s;%s;%s;%s;%s",
+		vcard_append_printf(&str, "ADR;TYPE=HOME,PREF:%s;%s;%s;%s;%s;%s;%s",
 			entry->address.post_office_box[0]  ? entry->address.post_office_box  : "",
 			entry->address.extended_address[0] ? entry->address.extended_address : "",
 			entry->address.street[0]           ? entry->address.street           : "",
@@ -102,57 +136,57 @@ GNOKII_API int gn_phonebook2vcard(FILE *f, gn_phonebook_entry *entry, char *loca
 		switch (entry->subentries[i].entry_type) {
 		case GN_PHONEBOOK_ENTRY_Email:
 			add_slashes(name, entry->subentries[i].data.number, sizeof(name), strlen(entry->subentries[i].data.number));
-			gn_vcard_fprintf(f, "EMAIL;TYPE=INTERNET:%s", name);
+			vcard_append_printf(&str, "EMAIL;TYPE=INTERNET:%s", name);
 			break;
 		case GN_PHONEBOOK_ENTRY_Postal:
 			add_slashes(name, entry->subentries[i].data.number, sizeof(name), strlen(entry->subentries[i].data.number));
-			gn_vcard_fprintf(f, "ADR;TYPE=HOME:%s", name);
+			vcard_append_printf(&str, "ADR;TYPE=HOME:%s", name);
 			break;
 		case GN_PHONEBOOK_ENTRY_Note:
 			add_slashes(name, entry->subentries[i].data.number, sizeof(name), strlen(entry->subentries[i].data.number));
-			gn_vcard_fprintf(f, "NOTE:%s", name);
+			vcard_append_printf(&str, "NOTE:%s", name);
 			break;
 		case GN_PHONEBOOK_ENTRY_Number:
 			switch (entry->subentries[i].number_type) {
 			case GN_PHONEBOOK_NUMBER_Home:
-				gn_vcard_fprintf(f, "TEL;TYPE=HOME:%s", entry->subentries[i].data.number);
+				vcard_append_printf(&str, "TEL;TYPE=HOME:%s", entry->subentries[i].data.number);
 				break;
 			case GN_PHONEBOOK_NUMBER_Mobile:
-				gn_vcard_fprintf(f, "TEL;TYPE=CELL:%s", entry->subentries[i].data.number);
+				vcard_append_printf(&str, "TEL;TYPE=CELL:%s", entry->subentries[i].data.number);
 				break;
 			case GN_PHONEBOOK_NUMBER_Fax:
-				gn_vcard_fprintf(f, "TEL;TYPE=FAX:%s", entry->subentries[i].data.number);
+				vcard_append_printf(&str, "TEL;TYPE=FAX:%s", entry->subentries[i].data.number);
 				break;
 			case GN_PHONEBOOK_NUMBER_Work:
-				gn_vcard_fprintf(f, "TEL;TYPE=WORK:%s", entry->subentries[i].data.number);
+				vcard_append_printf(&str, "TEL;TYPE=WORK:%s", entry->subentries[i].data.number);
 				break;
 			case GN_PHONEBOOK_NUMBER_None:
 			case GN_PHONEBOOK_NUMBER_Common:
 			case GN_PHONEBOOK_NUMBER_General:
-				gn_vcard_fprintf(f, "TEL;TYPE=VOICE:%s", entry->subentries[i].data.number);
+				vcard_append_printf(&str, "TEL;TYPE=VOICE:%s", entry->subentries[i].data.number);
 				break;
 			default:
-				gn_vcard_fprintf(f, "TEL;TYPE=X-UNKNOWN-%d: %s", entry->subentries[i].number_type, entry->subentries[i].data.number);
+				vcard_append_printf(&str, "TEL;TYPE=X-UNKNOWN-%d: %s", entry->subentries[i].number_type, entry->subentries[i].data.number);
 				break;
 			}
 			break;
 		case GN_PHONEBOOK_ENTRY_URL:
 			add_slashes(name, entry->subentries[i].data.number, sizeof(name), strlen(entry->subentries[i].data.number));
-			gn_vcard_fprintf(f, "URL:%s", name);
+			vcard_append_printf(&str, "URL:%s", name);
 			break;
 		case GN_PHONEBOOK_ENTRY_JobTitle:
 			add_slashes(name, entry->subentries[i].data.number, sizeof(name), strlen(entry->subentries[i].data.number));
-			gn_vcard_fprintf(f, "TITLE:%s", name);
+			vcard_append_printf(&str, "TITLE:%s", name);
 			break;
 		case GN_PHONEBOOK_ENTRY_Company:
 			add_slashes(name, entry->subentries[i].data.number, sizeof(name), strlen(entry->subentries[i].data.number));
-			gn_vcard_fprintf(f, "ORG:%s", name);
+			vcard_append_printf(&str, "ORG:%s", name);
 			break;
 		case GN_PHONEBOOK_ENTRY_Nickname:
-			gn_vcard_fprintf(f, "NICKNAME:%s", entry->subentries[i].data.number);
+			vcard_append_printf(&str, "NICKNAME:%s", entry->subentries[i].data.number);
 			break;
 		case GN_PHONEBOOK_ENTRY_Birthday:
-			gn_vcard_fprintf(f, "BDAY:%s", entry->subentries[i].data.number);
+			vcard_append_printf(&str, "BDAY:%s", entry->subentries[i].data.number);
 			break;
 		case GN_PHONEBOOK_ENTRY_Ringtone:
 		case GN_PHONEBOOK_ENTRY_Pointer:
@@ -166,16 +200,30 @@ GNOKII_API int gn_phonebook2vcard(FILE *f, gn_phonebook_entry *entry, char *loca
 			break;
 		default:
 			add_slashes(name, entry->subentries[i].data.number, sizeof(name), strlen(entry->subentries[i].data.number));
-			gn_vcard_fprintf(f, "X-GNOKII-%d: %s", entry->subentries[i].entry_type, name);
+			vcard_append_printf(&str, "X-GNOKII-%d: %s", entry->subentries[i].entry_type, name);
 			break;
 		}
 	}
 
-	gn_vcard_fprintf(f, "END:VCARD");
+	vcard_append_printf(&str, "END:VCARD");
 	/* output an empty line */
-	gn_vcard_fprintf(f, "");
+	vcard_append_printf(&str, "");
 
-	return 0;
+	return str.str;
+}
+
+GNOKII_API int gn_phonebook2vcard(FILE *f, gn_phonebook_entry *entry, char *location)
+{
+	char *vcard;
+	int retval;
+
+	vcard = gn_phonebook2vcardstr (entry);
+	if (vcard == NULL)
+		return -1;
+	retval = fputs (vcard, f);
+	free (vcard);
+
+	return retval;
 }
 
 #define BEGINS(a) ( !strncmp(buf, a, strlen(a)) )
@@ -197,15 +245,35 @@ GNOKII_API int gn_phonebook2vcard(FILE *f, gn_phonebook_entry *entry, char *loca
 #undef ERROR
 #define ERROR(a) fprintf(stderr, "%s\n", a)
 
-/* We assume gn_phonebook_entry is ready for writing in, ie. no rubbish inside */
+static void str_append_printf(vcard_string *str, const char *s)
+{
+	int len;
+
+	if (str->str == NULL) {
+		str->str = strdup (s);
+		str->len = strlen (s) + 1;
+		return;
+	}
+
+	len = strlen (s);
+	str->str = realloc(str->str, len + str->len);
+
+	memcpy (str->str + str->len - 1, s, len);
+	str->len = str->len + len;
+	str->end = str->str + str->len;
+
+	/* NULL-terminate the string now */
+	*(str->end - 1) = '\0';
+}
+
 GNOKII_API int gn_vcard2phonebook(FILE *f, gn_phonebook_entry *entry)
 {
 	char buf[1024];
-	char memloc[10];
-	char memory_name[10];
+	vcard_string str;
+	int retval;
+
+	memset(&str, 0, sizeof(str));
 
-	memset(memloc, 0, 10);
-	memset(memory_name, 0, 10);
 	while (1) {
 		if (!fgets(buf, 1024, f))
 			return -1;
@@ -213,33 +281,67 @@ GNOKII_API int gn_vcard2phonebook(FILE *f, gn_phonebook_entry *entry)
 			break;
 	}
 
-	while (1) {
-		int line_len = 0;
-		int c = 0;
+	str_append_printf (&str, "BEGIN:VCARD\r\n");
+	while (fgets(buf, 1024, f)) {
+		str_append_printf (&str, buf);
+		if (BEGINS("END:VCARD"))
+			break;
+	}
 
-		do {
-			if (!fgets(buf + line_len, 1024 - line_len, f)) {
-				ERROR(_("Vcard began but not ended?"));
-				return -1;
-			}
-			/* There's either "\n" or "\r\n' sequence at the
-			 * end of the line */
-			line_len = strlen(buf);
-			if (buf[line_len - 1] == '\n') {
-				buf[--line_len] = 0;
-			}
-			if (!line_len)
-				continue;
-			if (buf[line_len - 1] == '\r') {
-				buf[--line_len] = 0;
-			}
-			if (!line_len)
-				continue; 
-			/* perform line unfolding (see RFC2425) */
-			c = fgetc(f);
-		} while ((c == ' ') || (c == 0x09));
-		/* ungetc() is EOF safe */
-		ungetc(c, f);
+	retval = gn_vcardstr2phonebook (str.str, entry);
+	free (str.str);
+
+	return retval;
+}
+
+/* We assume gn_phonebook_entry is ready for writing in, ie. no rubbish inside */
+GNOKII_API int gn_vcardstr2phonebook(const char *vcard, gn_phonebook_entry *entry)
+{
+	char memloc[10];
+	char memory_name[10];
+	char *v, *fold, *s, **lines;
+	int num_lines, i;
+
+	if (vcard == NULL)
+		return -1;
+
+	memset(memloc, 0, sizeof(memloc));
+	memset(memory_name, 0, sizeof(memory_name));
+
+	/* Remove folding */
+	v = strdup (vcard);
+	fold = strstr (v, "\n ");
+	while (fold != NULL) {
+		memmove (fold, fold + 2, strlen (fold) - 2);
+		fold = strstr (fold, "\n ");
+	}
+	fold = strstr (v, "\n\t");
+	while (fold != NULL) {
+		memmove (fold, fold + 2, strlen (fold) - 2);
+		fold = strstr (v, "\n\t");
+	}
+
+	/* Count the number of lines */
+	s = strstr (v, "\n");
+	for (num_lines = 0; s != NULL; num_lines++) {
+		s = strstr (s + 1, "\n");
+	}
+
+	lines = gnokii_strsplit (v, "\n", num_lines);
+
+	for (i = 0; i < num_lines; i++) {
+		char *buf;
+		int line_len;
+
+		if (lines[i] == NULL || *lines[i] == '\0')
+			continue;
+
+		buf = lines[i];
+		line_len = strlen (buf);
+
+		/* Strip traling '\r's */
+		while (line_len > 0 && buf[line_len-1] == '\r')
+			buf[--line_len] = '\0';
 
 		if (BEGINS("N:")) {
 			if (0 < sscanf(buf +2 , "%64[^;];%64[^;];%64[^;];%64[^;];%64[^;]\n",
@@ -287,5 +389,7 @@ GNOKII_API int gn_vcard2phonebook(FILE *f, gn_phonebook_entry *entry)
 		if (BEGINS("END:VCARD"))
 			break;
 	}
+	
 	return 0;
 }
+
