@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-monitor.c,v 1.16 2008/05/30 17:09:14 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -157,7 +157,7 @@ static void displaycall(int call_id)
 		break;
 	}
 
-	fprintf(stderr, _("CALL%d: %s %s(%s) (callid: %d, duration: %d sec)\n"),
+	fprintf(stdout, _("CALL%d: %s %s (%s) (callid: %d, duration: %d sec)\n"),
 		call_id, s, call->remote_number, call->remote_name,
 		call->call_id, (int)delta.tv_sec);
 #endif
@@ -245,6 +245,7 @@ gn_error monitormode(int argc, char *argv[], gn_data *data, struct gn_statemachi
 	cb_widx = 0;
 	gn_sm_functions(GN_OP_SetCellBroadcast, data, state);
 
+	memset(&networkinfo, 0, sizeof(gn_network_info));
 	if (gn_sm_functions(GN_OP_GetNetworkInfo, data, state) == GN_ERR_NONE) {
 		fprintf(stdout, _("Network: %s, %s (%s)\n"),
 			gn_network_name_get(networkinfo.network_code), gn_country_name_get(networkinfo.network_code),
@@ -480,11 +481,11 @@ gn_error getdisplaystatus(gn_data *data, struct gn_statemachine *state)
 }
 
 /* Displays usage of --netmonitor command */
-void netmonitor_usage(FILE *f, int exitval)
+int netmonitor_usage(FILE *f, int exitval)
 {
 	fprintf(f, _(" usage: --netmonitor {reset|off|field|devel|next|nr}\n"
 		));
-	exit(exitval);
+	return exitval;
 }
 
 gn_error netmonitor(char *m, gn_data *data, struct gn_statemachine *state)
@@ -506,7 +507,7 @@ gn_error netmonitor(char *m, gn_data *data, struct gn_statemachine *state)
 	else {
 		mode = gnokii_atoi(m);
 		if (errno || mode < 1 || mode >= 0xf0)
-			netmonitor_usage(stderr, -1);
+			return netmonitor_usage(stderr, -1);
 	}
 
 	nm.field = mode;
