@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-sms.c,v 1.55 2008/10/05 10:36:25 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -120,6 +120,26 @@ int sendsms_usage(FILE *f, int exitval)
 	return exitval;
 }
 
+gn_gsm_number_type get_number_type(const char *number)
+{
+	gn_gsm_number_type type;
+
+	if (!number)
+		return GN_GSM_NUMBER_Unknown;
+	if (*number == '+') {
+		type = GN_GSM_NUMBER_International;
+		number++;
+	} else {
+		type = GN_GSM_NUMBER_Unknown;
+	}
+	while (*number) {
+		if (!isdigit(*number))
+			return GN_GSM_NUMBER_Alphanumeric;
+		number++;
+	}
+	return type;
+}
+
 /* Send SMS messages. */
 gn_error sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 {
@@ -127,7 +147,7 @@ gn_error sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 	gn_error error;
 	/* The maximum length of an uncompressed concatenated short message is
 	   255 * 153 = 39015 default alphabet characters */
-	int input_len, i, curpos = 0;
+	int i, curpos = 0;
 
 	struct option options[] = {
 		{ "smsc",      required_argument, NULL, '1' },
@@ -144,26 +164,21 @@ gn_error sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 		{ NULL,        0,                 NULL, 0 }
 	};
 
-	input_len = GN_SMS_MAX_LENGTH;
-
 	/* The memory is zeroed here */
 	gn_sms_default_submit(&sms);
 
-	memset(&sms.remote.number, 0, sizeof(sms.remote.number));
 	snprintf(sms.remote.number, sizeof(sms.remote.number), "%s", optarg);
-	if (sms.remote.number[0] == '+')
-		sms.remote.type = GN_GSM_NUMBER_International;
-	else
-		sms.remote.type = GN_GSM_NUMBER_Unknown;
+	sms.remote.type = get_number_type(sms.remote.number);
+	if (sms.remote.type == GN_GSM_NUMBER_Alphanumeric) {
+		dprintf("Invalid phone number\n");
+		return GN_ERR_WRONGDATAFORMAT;
+	}
 
 	while ((i = getopt_long(argc, argv, "l:rv:C:8ia:o:w:", options, NULL)) != -1) {
 		switch (i) {       /* -c for compression. not yet implemented. */
 		case '1': /* SMSC number */
 			snprintf(sms.smsc.number, sizeof(sms.smsc.number), "%s", optarg);
-			if (sms.smsc.number[0] == '+')
-				sms.smsc.type = GN_GSM_NUMBER_International;
-			else
-				sms.smsc.type = GN_GSM_NUMBER_Unknown;
+			sms.smsc.type = get_number_type(sms.smsc.number);
 			break;
 
 		case '2': /* SMSC number index in phone memory */
@@ -181,13 +196,7 @@ gn_error sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 			break;
 
 		case 'l': /* we send long message */
-			input_len = gnokii_atoi(optarg);
-			if (errno || input_len < 0)
-				return sendsms_usage(stderr, -1);
-			if (input_len > 255 * GN_SMS_MAX_LENGTH) {
-				fprintf(stderr, _("Input too long! (%d, maximum is %d)\n"), input_len, 255 * GN_SMS_MAX_LENGTH);
-				return GN_ERR_INVALIDSIZE;
-			}
+			/* ignored. Left for compatibility */
 			break;
 
 		case 'a': /* Animation */ {
@@ -251,13 +260,12 @@ gn_error sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 
 		case '8':
 			sms.dcs.u.general.alphabet = GN_SMS_DCS_8bit;
-			input_len = GN_SMS_8BIT_MAX_LENGTH ;
 			break;
 
 		case 'i':
 			sms.user_data[0].type = GN_SMS_DATA_iMelody;
 			sms.user_data[1].type = GN_SMS_DATA_None;
-			error = readtext(&sms.user_data[0], input_len);
+			error = readtext(&sms.user_data[0]);
 			if (error != GN_ERR_NONE)
 				return error;
 			if (sms.user_data[0].length < 1) {
@@ -273,7 +281,7 @@ gn_error sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 
 			if (!optarg || strlen(optarg) > 255) {
 				fprintf(stderr, _("URL is too long (max 255 chars). Quitting.\n"));
-			        return GN_ERR_INVALIDSIZE;
+				return GN_ERR_INVALIDSIZE;
 			}
 
 			sms.user_data[curpos].type = GN_SMS_DATA_WAPPush;
@@ -327,12 +335,9 @@ gn_error sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 	if (!sms.smsc.type) sms.smsc.type = GN_GSM_NUMBER_Unknown;
 
 	if (curpos != -1) {
-		error = readtext(&sms.user_data[curpos], input_len);
-		if (error != GN_ERR_NONE) return error;
-		if (sms.user_data[curpos].length < 1) {
-			fprintf(stderr, _("Empty message. Quitting.\n"));
-			return GN_ERR_INVALIDSIZE;
-		}
+		error = readtext(&sms.user_data[curpos]);
+		if (error != GN_ERR_NONE)
+			return error;
 		sms.user_data[curpos].type = GN_SMS_DATA_Text;
 		if (!gn_char_def_alphabet(sms.user_data[curpos].u.text))
 			sms.dcs.u.general.alphabet = GN_SMS_DCS_UCS2;
@@ -344,11 +349,19 @@ gn_error sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 	/* Send the message. */
 	error = gn_sms_send(data, state);
 
-	if (error == GN_ERR_NONE)
-		fprintf(stderr, _("Send succeeded with reference %d!\n"), sms.reference);
-	else
+	if (error == GN_ERR_NONE) {
+		if (sms.parts > 1) {
+			int j;
+			fprintf(stderr, _("Message sent in %d parts with reference numbers:"), sms.parts);
+			for (j = 0; j < sms.parts; j++)
+				fprintf(stderr, " %d", sms.reference[j]);
+			fprintf(stderr, "\n");
+		} else
+			fprintf(stderr, _("Send succeeded with reference %d!\n"), sms.reference[0]);
+	} else
 		fprintf(stderr, _("SMS Send failed (%s)\n"), gn_error_print(error));
 
+	free(sms.reference);
 	return error;
 }
 
@@ -385,7 +398,7 @@ gn_error savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 {
 	gn_sms sms;
 	gn_error error = GN_ERR_INTERNALERROR;
-	int input_len, i;
+	int i;
 	char tmp[3];
 #if 0
 	int confirm = -1;
@@ -431,17 +444,13 @@ gn_error savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 	strcpy(sms.remote.number, "0");
 	sms.remote.type = GN_GSM_NUMBER_International;
 	sms.number = 0;
-	input_len = GN_SMS_MAX_LENGTH ;
 
 	/* Option parsing */
 	while ((i = getopt_long(argc, argv, "l:rsf:dt:", options, NULL)) != -1) {
 		switch (i) {
 		case '0': /* SMSC number */
-			snprintf(sms.smsc.number, sizeof(sms.smsc.number) - 1, "%s", optarg);
-			if (sms.smsc.number[0] == '+')
-				sms.smsc.type = GN_GSM_NUMBER_International;
-			else
-				sms.smsc.type = GN_GSM_NUMBER_Unknown;
+			snprintf(sms.smsc.number, sizeof(sms.smsc.number), "%s", optarg);
+			sms.smsc.type = get_number_type(sms.smsc.number);
 			break;
 		case '1': /* SMSC number index in phone memory */
 			data->message_center = calloc(1, sizeof(gn_sms_message_center));
@@ -457,11 +466,8 @@ gn_error savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 			free(data->message_center);
 			break;
 		case '2': /* sender number */
-			if (*optarg == '+')
-				sms.remote.type = GN_GSM_NUMBER_International;
-			else
-				sms.remote.type = GN_GSM_NUMBER_Unknown;
 			snprintf(sms.remote.number, GN_BCD_STRING_MAX_LENGTH, "%s", optarg);
+			sms.remote.type = get_number_type(sms.remote.number);
 			break;
 		case 'l': /* location to write to */
 			sms.number = gnokii_atoi(optarg);
@@ -563,7 +569,7 @@ gn_error savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 
 	if (!sms.smsc.type) sms.smsc.type = GN_GSM_NUMBER_Unknown;
 
-	error = readtext(&sms.user_data[0], input_len);
+	error = readtext(&sms.user_data[0]);
 	if (error != GN_ERR_NONE)
 		return error;
 	if (sms.user_data[0].length < 1) {
@@ -628,6 +634,8 @@ gn_error getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *s
 	gn_sms message;
 	char *memory_type_string;
 	int start_message, end_message, count, mode = 1;
+	int folder_count = -1;
+	int messages_read = 0;
 	char filename[64];
 	gn_error error = GN_ERR_NONE;
 	gn_bmp bitmap;
@@ -635,6 +643,8 @@ gn_error getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *s
 	char ans[5];
 	struct stat buf;
 	char *message_text;
+	bool cont = true;
+	bool all = false;
 
 	struct option options[] = {
 		{ "delete",     no_argument,       NULL, 'd' },
@@ -658,6 +668,32 @@ gn_error getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *s
 	if (errno || end_message < 0)
 		return getsms_usage(stderr, -1);
 
+	/* Let's determine number of messages in given folder */
+	if (end_message == INT_MAX) {
+		gn_error e;
+		int j;
+
+		all = true;
+		memset(&folderlist, 0, sizeof(folderlist));
+		gn_data_clear(data);
+		data->sms_folder_list = &folderlist;
+
+		e = gn_sm_functions(GN_OP_GetSMSFolders, data, state);
+
+		if (e == GN_ERR_NONE) {
+			for (j = 0; j < folderlist.number; j++) {
+				data->sms_folder = folderlist.folder + j;
+				if (folderlist.folder_id[j] == gn_str2memory_type(memory_type_string)) {
+					e = gn_sm_functions(GN_OP_GetSMSFolderStatus, data, state);
+					if (e == GN_ERR_NONE) {
+						folder_count = folderlist.folder[j].number;
+						dprintf("Folder %s (%s) has %u messages\n", folderlist.folder[j].name, gn_memory_type2str(folderlist.folder_id[j]), folderlist.folder[j].number);
+					}
+				}
+			}
+		}
+	}
+
 	*filename = '\0';
 	/* parse all options (beginning with '-') */
 	while ((i = getopt_long(argc, argv, "f:F:a:d", options, NULL)) != -1) {
@@ -693,8 +729,10 @@ parsefile:
 	folder.folder_id = 0;
 	data->sms_folder = &folder;
 	data->sms_folder_list = &folderlist;
+	count = start_message;
+
 	/* Now retrieve the requested entries. */
-	for (count = start_message; count <= end_message; count++) {
+	while (cont) {
 		bool done = false;
 		int offset = 0;
 
@@ -703,9 +741,11 @@ parsefile:
 		message.number = count;
 		data->sms = &message;
 
+		dprintf("Getting message #%d from %s\n", count, memory_type_string);
 		error = gn_sms_get(data, state);
 		switch (error) {
 		case GN_ERR_NONE:
+			messages_read++;
 			message_text = NULL;
 			fprintf(stdout, _("%d. %s (%s)\n"), message.number, gn_sms_message_type2str(message.type), gn_sms_message_status2str(message.status));
 			switch (message.type) {
@@ -848,17 +888,22 @@ parsefile:
 			}
 			break;
 		default:
-			if ((error == GN_ERR_INVALIDLOCATION) && (end_message == INT_MAX) && (count > start_message))
-				return GN_ERR_NONE;
 			fprintf(stderr, _("GetSMS %s %d failed! (%s)\n"), memory_type_string, count, gn_error_print(error));
 			if (error == GN_ERR_INVALIDMEMORYTYPE)
 				fprintf(stderr, _("See the gnokii manual page for the supported memory types with the phone\nyou use.\n"));
-			if (error != GN_ERR_EMPTYLOCATION)
-				return error;
 			break;
 		}
 		if (mode == -1)
-			break;	}
+			cont = false;
+		if (count >= end_message)
+			cont = false;
+		if ((folder_count > 0) && (messages_read >= (folder_count - start_message + 1)))
+			cont = false;
+		/* Avoid infinite loops */
+		if (all && error != GN_ERR_NONE && error != GN_ERR_EMPTYLOCATION)
+			cont = false;
+		count++;
+	}
 
 	/* FIXME: We return the value of the last read.
 	 * What should we return?
@@ -1182,15 +1227,15 @@ gn_error showsmsfolderstatus(gn_data *data, struct gn_statemachine *state)
 		return error;
 	}
 
-	fprintf(stdout, _("No. Name                               Id #Msg\n"));
-	fprintf(stdout, _("==============================================\n"));
+	fprintf(stdout, _("No. Name                                         Id #Msg\n"));
+	fprintf(stdout, _("========================================================\n"));
 	for (i = 0; i < folders.number; i++) {
 		data->sms_folder = folders.folder + i;
 		if ((error = gn_sm_functions(GN_OP_GetSMSFolderStatus, data, state)) != GN_ERR_NONE) {
 			fprintf(stderr, _("Cannot stat folder \"%s\": %s\n"), folders.folder[i].name, gn_error_print(error));
 			return error;
 		}
-		fprintf(stdout, _("%3d %-32s %4s %4u\n"), i, folders.folder[i].name, gn_memory_type2str(folders.folder_id[i]), folders.folder[i].number);
+		fprintf(stdout, _("%3d %-42s %4s %4u\n"), i, folders.folder[i].name, gn_memory_type2str(folders.folder_id[i]), folders.folder[i].number);
 	}
 
 	return GN_ERR_NONE;
