@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-utils.c,v 1.11 2008/01/29 08:55:48 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -43,18 +43,15 @@
 
 #include <stdio.h>
 #include <signal.h>
+#ifdef HAVE_SYS_STAT_H
+#  include <sys/stat.h>
+#endif
 
 #include "gnokii-app.h"
 #include "gnokii.h"
 
-/* FIXME - this should not ask for confirmation here - I'm not sure what calls it though */
-/* mode == 0 -> overwrite
- * mode == 1 -> ask
- * mode == 2 -> append
- */
-int writefile(char *filename, char *text, int mode)
+int askoverwrite(const char *filename, int mode)
 {
-	FILE *file;
 	int confirm = -1;
 	char ans[5];
 	struct stat buf;
@@ -64,7 +61,7 @@ int writefile(char *filename, char *text, int mode)
 		fprintf(stdout, _("File %s exists.\n"), filename);
 		while (confirm < 0) {
 			fprintf(stdout, _("Overwrite? (yes/no) "));
-			gn_line_get(stdin, ans, 4);
+			gn_line_get(stdin, ans, sizeof(ans) - 1);
 			if (!strcmp(ans, _("yes")))
 				confirm = 1;
 			else if (!strcmp(ans, _("no")))
@@ -73,6 +70,19 @@ int writefile(char *filename, char *text, int mode)
 		if (!confirm)
 			return -1;
 	}
+	return mode;
+}
+
+/* FIXME - this should not ask for confirmation here - I'm not sure what calls it though */
+/* mode == 0 -> overwrite
+ * mode == 1 -> ask
+ * mode == 2 -> append
+ */
+int writefile(char *filename, char *text, int mode)
+{
+	FILE *file;
+
+	mode = askoverwrite(filename, mode);
 
 	/* append */
 	if (mode == 2)
@@ -91,7 +101,31 @@ int writefile(char *filename, char *text, int mode)
 	return 2;
 }
 
-gn_error readtext(gn_sms_user_data *udata, int input_len)
+int writebuffer(const char *filename, const char *buffer, size_t nitems, int mode)
+{
+	FILE *file;
+
+	mode = askoverwrite(filename, mode);
+
+	/* append */
+	if (mode == 2)
+		file = fopen(filename, "a");
+	/* overwrite */
+	else
+		file = fopen(filename, "w");
+
+	if (!file) {
+		fprintf(stderr, _("Can't open file %s for writing!\n"),  filename);
+		return -1;
+	}
+	if (fwrite(buffer, 1, nitems, file) != nitems)
+		return -1;
+	fclose(file);
+	/* Return value is used as new mode. Set it to append mode */
+	return 2;
+}
+
+gn_error readtext(gn_sms_user_data *udata)
 {
 	/* The maximum length of an uncompressed concatenated short message is
 	   255 * 153 = 39015 default alphabet characters */
@@ -112,8 +146,8 @@ gn_error readtext(gn_sms_user_data *udata, int input_len)
 	}
 	if (udata->type != GN_SMS_DATA_iMelody && chars_read > 0 && message_buffer[chars_read - 1] == '\n')
 		message_buffer[--chars_read] = 0;
-	if (chars_read > input_len || chars_read > sizeof(udata->u.text) - 1) {
-		fprintf(stderr, _("Input too long! (%d, maximum is %d)\n"), chars_read, input_len);
+	if (chars_read > (sizeof(udata->u.text) - 1)) {
+		fprintf(stderr, _("Input too long! (%d, maximum is %d)\n"), chars_read, (sizeof(udata->u.text) - 1));
 		return GN_ERR_INTERNALERROR;
 	}
 
