@@ -1,6 +1,6 @@
 /*
 
-  $Id: compat.h,v 1.64 2008/05/27 05:54:07 cfturkja Exp $
+  $Id$
 
   G N O K I I
 
@@ -252,11 +252,17 @@ time_t timegm(struct tm *tm);
 /* This one is for NLS. */
 #ifdef ENABLE_NLS
 #  include <libintl.h>
-#  define _(x) gettext(x)
-#  define N_(x) gettext_noop(x)
+#  ifdef COMPILING_LIBGNOKII
+#    define _(x) dgettext(GETTEXT_PACKAGE, x)
+#    define N_(x) dgettext_noop(GETTEXT_PACKAGE, x)
+#  else
+#    define _(x) gettext(x)
+#    define N_(x) gettext_noop(x)
+#  endif
 #else
 #  define _(x) (x)
 #  define N_(x) (x)
+#  define dgettext(y, x) (x)
 #endif /* ENABLE_NLS */
 
 /* Definitions for u8, u16 and u32, borrowed from
@@ -284,7 +290,7 @@ time_t timegm(struct tm *tm);
 #endif
 
 #ifndef HAVE_U_INT8_T
-	typedef u_int_8_t;
+	typedef unsigned char u_int8_t;
 #endif
 
 #ifdef HAVE_WCRTOMB
@@ -306,5 +312,21 @@ time_t timegm(struct tm *tm);
 #  endif
 #endif
 
+/* Bool type */
+#ifdef HAVE_STDBOOL_H
+#  include <stdbool.h>
+#else
+#  if !HAVE__BOOL
+#    ifdef __cplusplus
+	typedef bool _Bool;
+#    else
+	typedef unsigned char _Bool;
+#    endif
+#  endif
+#  define bool _Bool
+#  define false 0
+#  define true 1
+#  define __bool_true_false_are_defined 1
+#endif
 
 #endif
