@@ -1,6 +1,6 @@
 /*
 
-  $Id: common.h,v 1.158 2008/04/13 22:02:52 dforsi Exp $
+  $Id$
 
   G N O K I I
 
@@ -84,9 +84,12 @@ typedef enum {
 	GN_MT_TA, /* for compatibility only: TA=computer memory */
 	GN_MT_CB, /* Currently selected memory */
 	GN_MT_IN, /* Inbox for folder aware phones */
-	GN_MT_OU, /* Outbox  */
+	GN_MT_OU, /* Outbox, sent items */
 	GN_MT_AR, /* Archive */
 	GN_MT_TE, /* Templates */
+	GN_MT_SR, /* Status reports */
+	GN_MT_DR, /* Drafts */
+	GN_MT_OUS, /* Outbox, items to be sent */
 	GN_MT_F1, /* 1st CUSTOM FOLDER */
 	GN_MT_F2,
 	GN_MT_F3,
@@ -114,10 +117,13 @@ typedef enum {
 GNOKII_API gn_memory_type gn_str2memory_type(const char *s);
 GNOKII_API const char *gn_memory_type2str(gn_memory_type mt);
 
-/* Power source types */
+/* Power source types - see subclause 8.4 of ETSI TS 127 007 V8.6.0 (2009-01) */
 typedef enum {
-	GN_PS_ACDC = 1, /* AC/DC powered (charging) */
-	GN_PS_BATTERY   /* Internal battery */
+	GN_PS_UNKNOWN,		/* +CME ERROR: ... */
+	GN_PS_ACDC = 1,		/* +CBC: 1,... AC/DC powered (charging) */
+	GN_PS_BATTERY,		/* +CBC: 0,... Battery powered (not charging) */
+	GN_PS_NOBATTERY,	/* +CBC: 2,... No battery */
+	GN_PS_FAULT		/* +CBC: 3,... There is a power fault and calls are inhibited */
 } gn_power_source;
 
 GNOKII_API const char *gn_power_source2str(gn_power_source s);
@@ -667,7 +673,8 @@ typedef struct {
 
 /* Data structures for the call divert */
 typedef enum {
-	GN_CDV_Busy = 0x01,
+	GN_CDV_Unconditional = 0x00,
+	GN_CDV_Busy,
 	GN_CDV_NoAnswer,
 	GN_CDV_OutOfReach,
 	GN_CDV_NotAvailable,
@@ -853,12 +860,11 @@ typedef struct {
 	unsigned char *file;	/* file contents */
 } gn_file;
 
-#define GN_FILES_MAX_COUNT 1024
-
 typedef struct {
 	char path[512];
-	gn_file *files[GN_FILES_MAX_COUNT];
+	gn_file **files;
 	int file_count;
+	int size;
 } gn_file_list;
 
 #endif	/* _gnokii_common_h */
