@@ -1,6 +1,6 @@
 /*
 
-  $Id: misc.h,v 1.56 2007/12/04 19:27:35 hadess Exp $
+  $Id$
 
   G N O K I I
 
@@ -41,18 +41,6 @@
 
 /* Some general defines. */
 
-#ifndef false
-#  define false (0)
-#endif
-
-#ifndef true
-#  define true (!false)
-#endif
-
-#ifndef bool
-#  define bool int
-#endif
-
 #define ARRAY_LEN(x) (sizeof((x)) / sizeof((x)[0]))
 
 /* If glib.h is included, G_GNUC_PRINTF is already defined. */
@@ -116,6 +104,8 @@
 #define PM_AUTHENTICATION	0x0200
 #define PM_FOLDERS		0x0400
 #define PM_FULLPBK		0x0800
+#define PM_SMSFILE		0x1000
+
 /* This one indicated reported cases of breaking the phone by xgnokii
  * in FBUS orver IrDA mode */
 #define PM_XGNOKIIBREAKAGE	0x8000
