@@ -1,6 +1,6 @@
 /*
 
-  $Id: lowlevel.h,v 1.14 2008/01/27 20:17:04 ja Exp $
+  $Id$
 
   S M S D
 
@@ -34,6 +34,8 @@
 #include <glib.h>
 #include "gnokii.h"
 
+#define	MAX_CONSECUTIVE_ERRORS	5
+
 typedef enum {
   Event_SendSMSMessage,
   Event_DeleteSMSMessage,
@@ -51,7 +53,6 @@ typedef struct {
 } D_SMSMessage;
 
 typedef struct {
-//  bool working;
   struct {
     gchar *model;
     gchar *revision;
