@@ -1,6 +1,6 @@
 /*
 
-  $Id: config.h,v 1.5 2008/05/30 19:34:54 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -39,7 +39,7 @@
 #endif
 
 #undef VERSION
-#define VERSION "0.6.26"
+#define VERSION "0.6.28"
 
 /* We support Bluetooth and IRDA on MSVC8 */
 #define HAVE_BLUETOOTH 1
@@ -82,8 +82,8 @@
 #undef HAVE_TERMIOS_CSPEED
 
 /* Define both if you have ISO C99 compliant snprintf */
-#define HAVE_SNPRINTF
-#define HAVE_C99_SNPRINTF
+#undef HAVE_SNPRINTF
+#undef HAVE_C99_SNPRINTF
 
 /* Define both if you have ISO C99 compliant vsnprintf */
 #define HAVE_VSNPRINTF 1
